@@ -1,7 +1,12 @@
 #' Spread a key-value pair across multiple columns.
 #'
-#' @param key,value Bare (unquoted) names of key and value columns.
+#' @param key The bare (unquoted) name of the column whose values will be used
+#'   as column headings.
+#' @param value The bare (unquoted) name of the column whose values will
+#'  populate the cells.
 #' @inheritParams spread_
+#' @seealso \code{\link{spread_}} for a version that uses regular evaluation
+#'   and is suitable for programming with.
 #' @export
 #' @examples
 #' library(dplyr)
@@ -18,6 +23,13 @@
 #' # Spread and gather are complements
 #' df <- data.frame(x = c("a", "b"), y = c(3, 4), z = c(5, 6))
 #' df %>% spread(x, y) %>% gather(x, y, a:b, na.rm = TRUE)
+#'
+#' # Use 'convert = TRUE' to produce variables of mixed type
+#' df <- data.frame(row = rep(c(1, 51), each = 3),
+#'                  var = c("Sepal.Length", "Species", "Species_num"),
+#'                  value = c(5.1, "setosa", 1, 7.0, "versicolor", 2))
+#' df %>% spread(var, value) %>% str
+#' df %>% spread(var, value, convert = TRUE) %>% str
 spread <- function(data, key, value, fill = NA, convert = FALSE, drop = TRUE) {
   key_col <- col_name(substitute(key))
   value_col <- col_name(substitute(value))
@@ -31,17 +43,19 @@ spread <- function(data, key, value, fill = NA, convert = FALSE, drop = TRUE) {
 #'
 #' @param data A data frame.
 #' @param key_col,value_col Strings giving names of key and value cols.
-#' @param fill If set, missing values will be replaced with this value.
-#'   Note that there are two types of missingness in the input: explicit
-#'   missing values (i.e. \code{NA}), and implicit missings, rows that
-#'   simply aren't present. Both types of missing value will be replaced by
-#'   \code{fill}.
-#' @param convert If \code{TRUE}, \code{\link{type.convert}} with
-#'   \code{asis = TRUE} will be run on each of the new columns. This is
-#'   useful if the value column was a mix of variables that was coerced to
-#'   a string.
-#' @param drop If \code{FALSE}, will keep factor levels that don't appear
-#'   in the data, filling in missing combinations with \code{fill}.
+#' @param fill If set, missing values will be replaced with this value. Note
+#'   that there are two types of missingness in the input: explicit missing
+#'   values (i.e. \code{NA}), and implicit missings, rows that simply aren't
+#'   present. Both types of missing value will be replaced by \code{fill}.
+#' @param convert If \code{TRUE}, \code{\link{type.convert}} with \code{asis =
+#'   TRUE} will be run on each of the new columns. This is useful if the value
+#'   column was a mix of variables that was coerced to a string. If the class of
+#'   the value column was factor or date, note that will not be true of the new
+#'   columns that are produced, which are coerced to character before type
+#'   conversion.
+#' @param drop If \code{FALSE}, will keep factor levels that don't appear in the
+#'   data, filling in missing combinations with \code{fill}.
+#' @keywords internal
 #' @export
 spread_ <- function(data, key_col, value_col, fill = NA, convert = FALSE,
                     drop = TRUE) {
@@ -56,19 +70,26 @@ spread_ <- function(data, key_col, value_col, fill = NA, convert = FALSE,
 }
 
 #' @export
+#' @importFrom dplyr as_data_frame
 spread_.data.frame <- function(data, key_col, value_col, fill = NA,
                                convert = FALSE, drop = TRUE) {
 
   col <- data[key_col]
-  col_id <- dplyr::id(col, drop = drop)
+  col_id <- id(col, drop = drop)
   col_labels <- split_labels(col, col_id, drop = drop)
 
   rows <- data[setdiff(names(data), c(key_col, value_col))]
-  row_id <- dplyr::id(rows, drop = drop)
-  row_labels <- split_labels(rows, row_id, drop = drop)
-  rownames(row_labels) <- NULL
+  if (length(rows) == 0) {
+    # Special case when there's only one row
+    row_id <- structure(1L, n = 1L)
+    row_labels <- as.data.frame(matrix(nrow = 1, ncol = 0))
+  } else {
+    row_id <- id(rows, drop = drop)
+    row_labels <- split_labels(rows, row_id, drop = drop)
+    rownames(row_labels) <- NULL
+  }
 
-  overall <- dplyr::id(list(col_id, row_id), drop = FALSE)
+  overall <- id(list(col_id, row_id), drop = FALSE)
   n <- attr(overall, "n")
   # Check that each output value occurs in unique location
   if (anyDuplicated(overall)) {
@@ -94,14 +115,19 @@ spread_.data.frame <- function(data, key_col, value_col, fill = NA,
   if (!is.na(fill)) {
     ordered[is.na(ordered)] <- fill
   }
-  if (convert) {
-    ordered <- type.convert(ordered, as.is = TRUE)
-  }
 
+  if (convert && !is.character(ordered)) {
+    ordered <- as.character(ordered)
+  }
   dim(ordered) <- c(attr(row_id, "n"), attr(col_id, "n"))
   colnames(ordered) <- as.character(col_labels[[1]])
+  ordered <- matrixToDataFrame(ordered)
 
-  append_df(row_labels, matrixToDataFrame(ordered))
+  if (convert) {
+    ordered[] <- lapply(ordered, type.convert, as.is = TRUE)
+  }
+
+  append_df(row_labels, ordered)
 }
 
 #' @export
@@ -110,6 +136,12 @@ spread_.tbl_df <- function(data, key_col, value_col, fill = NA,
   dplyr::tbl_df(NextMethod())
 }
 
+#' @export
+spread_.grouped_df <- function(data, key_col, value_col, fill = NA,
+                               convert = FALSE, drop = TRUE) {
+  regroup(data, NextMethod(), except = c(key_col, value_col))
+}
+
 split_labels <- function(df, id, drop = TRUE) {
   if (length(df) == 0) {
     return(df)
